using System;
using System.Drawing;
using System.Windows.Forms;

namespace Panes {

	/// <summary>
	/// The FloatingPane will always be carrying one control in it -- and it will
	/// be of type PaneBase. This control of course will be carrying more than one
	/// control in it. 
	/// </summary>
	public partial class FloatingPane : Form {

		Timer closeTimer = new Timer();
	
		bool isFirstMousePeriod;

		private FloatingPane() {
		}

		public FloatingPane(Pane p, Form owner) {
			InitializeComponent();

			this.SetStyle(ControlStyles.EnableNotifyMessage, true);

			// no more than two levels of indirection
			if (owner.GetType() == typeof(FloatingPane))
				owner.Owner.AddOwnedForm(this);
			else
				owner.AddOwnedForm(this);

			//myContainer.AddPane(p, DockStyle.Fill, false);
			//this.Controls.Add(myContainer);

			//myContainer.DragEnabled = false;
			//PaneContainer.currentMovingPane = myContainer;

			p.Dock = DockStyle.Fill;
			p.IsRoot = true;

			PaneBase.CurrentMovingPane = p;
			this.Controls.Add(p);

			closeTimer.Enabled = false;
			closeTimer.Tick += new EventHandler(closeTimer_Tick);
			closeTimer.Interval = 25;


			this.ShowInTaskbar = false;

			PaneBase.PaneMoving = true;

			isFirstMousePeriod = true;
			this.Capture = true;
			this.Focus();

			// these don't work on clients!
			this.MouseMove += new MouseEventHandler(FloatingPane_MouseMove);
			this.MouseUp += new MouseEventHandler(FloatingPane_MouseUp);

			this.Move += new EventHandler(FloatingPane_Move);
			this.LostFocus += new EventHandler(FloatingPane_LostFocus);
		}

		void FloatingPane_LostFocus(object sender, EventArgs e) {
			if ( PaneBase.PaneMoving )
				this.Focus();
		}

		// as the form moves, keep notifying the panecontainers to check the mouse position
		void FloatingPane_Move(object sender, EventArgs e) {
			PaneBase.UpdateDragWindows();
		}


		#region Mouse moving only used for the first move

		void FloatingPane_MouseUp(object sender, MouseEventArgs e) {
			if ( e.Button == MouseButtons.Left ) {
				isFirstMousePeriod = false;
				PaneBase.PaneMoving = false;
			}
		}

		void FloatingPane_MouseMove(object sender, MouseEventArgs e) {
			if ( isFirstMousePeriod ) {
				Point mousePos = Control.MousePosition;
				mousePos.Offset(- (Width / 2), -SystemInformation.ToolWindowCaptionHeight / 2);
				//mousePos.Offset(-e.X - SystemInformation.FrameBorderSize.Width, -e.Y - SystemInformation.FrameBorderSize.Height);
				Location = mousePos;
			}
		}

		#endregion

		// this is a hack: can't call close from inside OnControlRemoved
		void closeTimer_Tick(object sender, EventArgs e) {
			Close();
			Dispose();
			closeTimer.Stop();
		}

		protected override void OnControlAdded(ControlEventArgs e) {
			base.OnControlAdded(e);
			e.Control.Dock = DockStyle.Fill;
		}

		protected override void OnControlRemoved(ControlEventArgs e) {
			base.OnControlRemoved(e);
			if (Controls.Count < 1)
				closeTimer.Start();
		}
		
		private const int WM_NCLBUTTONDOWN = 0x00A1;
		private const int WM_NCLBUTTONUP = 0x00A2;
		private const int WM_LBUTTONDOWN = 0x0201;
		private const int WM_LBUTTONUP = 0x0202;
		private const int WM_NCMOUSEMOVE = 0x00A0;

		private enum HitTestValues
		{
			HTERROR = -2,
			HTTRANSPARENT = -1,
			HTNOWHERE = 0,
			HTCLIENT = 1,
			HTCAPTION = 2,
			HTSYSMENU = 3,
			HTGROWBOX = 4,
			HTMENU = 5,
			HTHSCROLL = 6,
			HTVSCROLL = 7,
			HTMINBUTTON = 8,
			HTMAXBUTTON = 9,
			HTLEFT = 10,
			HTRIGHT = 11,
			HTTOP = 12,
			HTTOPLEFT = 13,
			HTTOPRIGHT = 14,
			HTBOTTOM = 15,
			HTBOTTOMLEFT = 16,
			HTBOTTOMRIGHT = 17,
			HTBORDER = 18,
			HTOBJECT = 19,
			HTCLOSE = 20,
			HTHELP = 21
		}

		


		// this is a nasty hack
		protected override void OnNotifyMessage(Message m){
			
			switch (m.Msg){

				// (usually) called at the end of a move, close enough I guess
				case WM_NCMOUSEMOVE:
					if ( !isFirstMousePeriod && (HitTestValues)m.WParam == HitTestValues.HTCAPTION ) {

						if ( PaneBase.PaneMoving && MouseButtons == MouseButtons.None ) {
							PaneBase.PaneMoving = false;
						}
					}
					break;

				// called on the start of a move
				case WM_NCLBUTTONDOWN:
					if ( !isFirstMousePeriod && (HitTestValues)m.WParam == HitTestValues.HTCAPTION ) {
						PaneBase.PaneMoving = true;
						PaneBase.CurrentMovingPane = ( Controls[0] as PaneBase );
					}
						
					break;
			}
		}
		
	}
}